
/**************************************************************************************************
*
*  SailingSensoreNode
*
*   Version:      1.0.0 - Februar 2009
*   Author:       Christoph Wartmann / chair for caad - ETH Z\u00fcrich  /  wartmann[at].arch.ethz.ch
*                 Etienne Ribeiro    / tutorial assistant caad      /  eribeiro[at]ethz.ch
*
*   Used Modules: modAneometer: Mesures wind speed
*                 modPressure: Mesures Pressure
*                 modStringSerial: Protocol to transmit values
*
*   Hardware:     Arduino Pro Mini
*                 XBee in RS232 Mode
*                 Pressure Sensor SCP 1000
*                 Aneometer Sensor using induction to calculate wind speed
*
**************************************************************************************************/


#include <EEPROM.h>


//
// Const

#include "WProgram.h"
void setup();
void loop();
void SerialEvent (char cmd, char* message);
int Aneo_getSpeedHZ (int digitalPin, int delayIntervall);
double Aneo_parseMS (int HZ);
double Aneo_parseKMH (int HZ);
int Aneo_getAverage ();
float Aneo_private_getAverage();
boolean scp1000_AppendSensor (byte drdy, byte csb, byte datain, byte dataout, byte sckclock, byte Operationmode);
boolean scp1000_startup();
boolean scp1000_checksumTest();
void scp1000_setOperationMode(int modeid);
long scp1000_Read(char which);
void scp1000_parseDegrees (int val, int *retSign, int *ret1, int *ret2);
long scp1000_parsePa (long val);
void scp1000_parsePa (long val, long *ret1, long *ret2);
void scp1000_SPI_write (byte registerByte, byte commandByte);
int scp1000_SPI_read8 (byte registerByte);
int scp1000_SPI_read16 (byte registerByte);
void Serial_Initialize (int boud);
void Serial_setID (char ID);
void Serial_sendMessage (char cmd, char* message);
void Serial_sendMessage (char cmd, char* msg1, char* msg2);
void Serial_sendMessage (char cmd, char* msg1, int msg2);
void Serial_sendMessage (char cmd, int message);
void Serial_sendMessage (char cmd, int message, int mode);
void Serial_sendMessage (char cmd, long message);
void Serial_check ();
int Serial_getLength (char *word);
int Serial_parseInteger (char* val);
long Serial_parseLong (char* val);
long Serial_pow (int x);
int NODE_ID = 16; // 0: not set, id from eeprom will be used (don't use id 10 or 13!)
//
char VERSION[] = "1.0";
#define aneometerSignalIn 9
#define ledPin 13


//
// Var

boolean startupDone;


//
// Setup

void setup() {
      
  // LED (2 * blink)
  pinMode(ledPin, OUTPUT);
  digitalWrite(ledPin, HIGH);
  delay(200);
  digitalWrite(ledPin, LOW);
  delay(200);
  digitalWrite(ledPin, HIGH);


  // NODE_ID to and from EEPROM
  if (NODE_ID == 0)
    NODE_ID = EEPROM.read(10);
  if (EEPROM.read(10) != NODE_ID)
    EEPROM.write (10, NODE_ID);
  
  
  // Serial
  Serial_Initialize (9600);
  Serial_setID (NODE_ID); // ID will be sent
  Serial_sendMessage ('m', "Setup begin");
  Serial_sendMessage ('m', "Firmware Verison ", VERSION);
  Serial_sendMessage ('m', "My ID ", NODE_ID);
  if (NODE_ID == 10 || NODE_ID == 13)
    Serial_sendMessage ('m', " CHANGE ID! ID 10 or 13 is not recommended to use!", VERSION);
  
  
  // Pressure
  //  -> Operationmode: 0: High speed (15bit, 9Hz), 1: High resolution (17bit, 1.8Hz), 2: Ultra low power (15bit, 1Hz), [3 Low poser but triggerd (15bit to 17bit, 1.8Hz) not supported on Sparkfun Breakout]
  byte operationmode = 1;
  startupDone = true;
  if (scp1000_AppendSensor (10, 11, 12, 13, 14, operationmode) == false) {
  
    Serial_sendMessage ('m', "Failed to startup Pressure Sensor");
    startupDone = false;
  
  }
  Serial_sendMessage ('m', "Setup done");

}




//
// Loop

void loop() {

  //
  // Aneometer
  
  
  // Mesure
  // (get Hz)
  int HzCounter = Aneo_getSpeedHZ (aneometerSignalIn, 100);
  // (Parse to m/s and km/h)
  //double ms = Aneo_parseMS (HzCounter);
  Serial_sendMessage ('a', HzCounter);    
  //double kmh = Aneo_parseKMH (HzCounter);
  // (get average Hz)
  //int HzAverage = Aneo_getAverage ();
  //double kmhAverage = Aneo_parseKMH (HzAverage);
  
  
  /*
  // Print
  if (HzCounter > 0) {
    
    // (Hz)
    Serial.print(HzCounter);
    Serial.print(" Hz sec.");
    Serial.print("       ");
    // (m/s)
    Serial.print((int)ms);
    Serial.print(".");
    Serial.print( int(abs((ms - (int)ms) * 100)) );
    Serial.print(" m/s");
    Serial.print("       ");
    // (km/h)
    Serial.print((int)kmh);
    Serial.print(".");
    Serial.print( int(abs((kmh - (int)kmh) * 100)) );
    Serial.print(" km/h");
    Serial.print("       ");
    // 
    Serial.print("Average: ");
    Serial.print((int)kmhAverage);
    Serial.print(".");
    Serial.print( int(abs((kmhAverage - (int)kmhAverage) * 100)) );
    Serial.print(" km/h");
    //
    Serial.println("");
    
  }
*/

  //
  // Pressure
  
  if (startupDone) {
    
    // Temperature
    int sign;
    int temp1;
    int temp2;
    scp1000_parseDegrees(scp1000_Read('t'), &sign, &temp1, &temp2);
    long temperature = sign * (temp1 * 100 + temp2);
    Serial_sendMessage ('t', temperature);
    long pressure = scp1000_parsePa(scp1000_Read('p'));
    Serial_sendMessage ('p', pressure);    
 
  }
  
   
  // Take a break
  delay(100);
  
}





// SerialEvent

void SerialEvent (char cmd, char* message) {
  
  
}


//
// Aneometer (Windsensor)
// Christoph Wertmann, Etienne Ribeiro
// 3. Februar 2009
//
//
// Pin
// green: Digital Input
// white: GND
// broun: 3V to 5V
//




// Var
// (average)
int sensordataCount = 0;
const int  sensordataBounderies = 10;
int sensordataHistory[sensordataBounderies];
int lastAverageValue = 0;
// (calibration)
float sensorCalibration = 14; // 35hz = 2.5m/s   // 1m/s = 14  61 == 8.4





//
// Aneo_getSpeedHZ
//  Mesures over a periode and returns Hz (Intervals / 1000ms)
//  digitalPin: Pin to mesure from
//  delayIntervall: Interval to mesure

int Aneo_getSpeedHZ (int digitalPin, int delayIntervall) {
  
  // 
  pinMode(digitalPin,INPUT);
  int HzCounter = 0;
  int lastImp = 0;


  // Mesure over a periode (delayIntervall)
  long start = millis();
  while (start + delayIntervall >= millis()) {
  
    int imp = pulseIn(digitalPin, HIGH);
    if(imp != lastImp) {

      HzCounter++;        // count HIGH impulse
      lastImp = max(imp,0);
       
    }

  }


  // Calculate Hz
  int Hz = HzCounter * (1000 / delayIntervall);
  
  
  // Average mesurements
  if (sensordataCount + 1 >= sensordataBounderies) {

    lastAverageValue = Aneo_private_getAverage();
    sensordataCount = 0;
  
  }
  sensordataHistory[sensordataCount] = Hz;
  sensordataCount ++;
  
      
  // Return Hz (HzCounter per 1000 ms)
  return Hz;
  
}


 
 
 
 
//
// Aneo_parse
//   Parse Hz to m/s or km/h

double Aneo_parseMS (int HZ) {
  
  // Calculate m/s from Hz
  return (HZ / sensorCalibration);

}

double Aneo_parseKMH (int HZ) {

  // Calculate km/h from Hz
  double ms = (HZ / sensorCalibration);
  return ms*3.6;

}





//
// Aneo_getAverage

int Aneo_getAverage () {
  
  return lastAverageValue;
  
}
float Aneo_private_getAverage() {
  
  float result = 0;
  for(int i = 0; i<sensordataBounderies; i++) {
    result += sensordataHistory[i];
  }
  result = result / sensordataBounderies;
  return result;
  
} 


/**************************************************************************************************
*
*  Pressure SCP1000
*
*   Version:      1.0.0 - Februar 2009
*   Author:       Christoph Wartmann / chair for caad - ETH Z\u00fcrich  /  wartmann[at].arch.ethz.ch
*                 Etienne Ribeiro    / tutorial assistant caad      /  eribeiro[at]ethz.ch
*
*   Desc:         This Module makes it easy to run the SCP1000 Pressure Sensor in your projects.
*
*   Protocol:     SPI
*
*   Voltage:      typical 2.7V, max 3.3V
*
*   Features implemented:
*                 - SPI Protocol
*                 - all supported Operationmodes
*
*   Not yet implemented:
*                 - I2C (supported on Sparkfun Breakout; Software addresses!)
*                 - self test
*                 - [Operationmode 4 = Triggered Mode (Not supported on Sparkfun Breakout)]
*                 - Standby Mode
*                 - Interrupt
*                 - Reset
*
*  Pins:          use any digital pin. example:
*                 DRDY = 24; (optional)
*                 CSB = 25;
*                 DATAIN (MISO) = 26; 
*                 DATAOUT (MOSI) = 27;
*                 SCK = 28;
*
*  Methodes:      boolean scp1000_AppendSensor (byte drdy, byte csb, byte datain, byte dataout, byte sckclock, byte Operationmode)
*                   drdy, csb, datain, dataout, sckclock: digital Pins, drdy is not needed and can be set to -1
*                   Operationmode:  0 = High speed (15bit, 9Hz), 1 = High resolution (17bit, 1.8Hz), 2 = Ultra low power (15bit, 1Hz)
*                   return false, if sensor coud not be initialized.
*                 void scp1000_setOperationMode(int modeid) 
*                   change operation mode
*                   modeid:  0 = High speed (15bit, 9Hz), 1 = High resolution (17bit, 1.8Hz), 2 = Ultra low power (15bit, 1Hz)
*                 long scp1000_Read(char which)
*                   'p' = get pressure
*                   't' = get temperature
*                   's'= get status
*                   'o' = get operation mode
*                   'n' = get operation status
*                 void scp1000_parseDegrees (int val, int *retSign, int *ret1, int *ret2)
*                   Parse value to degrees. val = scp1000_Read('t') and scp1000_parseDegrees (val,...)
*                   *retSign: Return Value (sign) -1 or 1. Access this value by &var
*                   *ret1: Return Value (upper). Access this value by &var
*                   *ret2: Return Value (lower). Access this value by &var
*                 long scp1000_parsePa (long val)
*                   Parse value to Pascal. val = scp1000_Read('p') and scp1000_parsePa (val)
*                 void scp1000_parsePa (long val, long *ret1, long *ret2)
*                   Parse value to Pascal. val = scp1000_Read('p') and scp1000_parsePa (val,...)
*                   *ret1: Return Value (upper). Access this value by &var
*                   *ret2: Return Value (lower). Access this value by &var
*                 boolean scp1000_printStatus (byte state) 
*                   Print current state of sensor. state = scp1000_Read('s') and scp1000_printStatus (state)                   
*                 boolean scp1000_printOperation (byte operation)
*                   Print operation mode. operationmode = scp1000_Read('o') and scp1000_printOperation (operationmode)                   
*
***************************************************************************************************/ 






//
// Const

// direct access:
#define Reg_REVID 0x00
#define Reg_DATAWR 0x01
#define Reg_ADDPTR 0x02
#define Reg_OPERATION 0x03
byte const Reg_OPERATION_modes[] = {0x09, 0x0A, 0x0B, 0x0C};  // (0x0C or TRIG pin)
#define Reg_OPSTATUS 0x04
#define Reg_RSTR 0x06
#define Reg_STATUS 0x07
byte const Reg_STATUS_startupbit = 0;
#define Reg_DATARD8 0x1F
byte const Reg_DATARD8_checksumerror = 0x00;
#define Reg_DATARD16 0x20 // 16bit
#define Reg_TEMPOUT 0x21 // 16bit
// indirect access:
#define Reg_CFG 0x00
#define Reg_TWIADD 0x05
#define Reg_USERDATA1 0x29
#define Reg_USERDATA2 0x2A
#define Reg_USERDATA3 0x2B
#define Reg_USERDATA4 0x2C


// more consts
int temp_factor = 20;
int pressure_factor = 4;


//
// Var

int scp1000_DRDY;        // 24 1: availability of new measurement
int scp1000_CSB;         // 25 selects the chip on multi-chip SPI bus
int scp1000_DATAIN;      // 26 MISO (Master In Slave Out)
int scp1000_DATAOUT;     // 27 MOSI (Master Out Slave In)
int scp1000_SCKCLOCK;    // 28 Clock




// *************************
//
//  Functions
//


//
// scp1000_AppendSensor
//  -> Operationmode: 0: High speed (15bit, 9Hz), 1: High resolution (17bit, 1.8Hz), 2: Ultra low power (15bit, 1Hz), [3 Low poser but triggerd (15bit to 17bit, 1.8Hz) not supported on Sparkfun Breakout]
boolean scp1000_AppendSensor (byte drdy, byte csb, byte datain, byte dataout, byte sckclock, byte Operationmode) {
  
  // Variables
  scp1000_DRDY = drdy;
  scp1000_CSB = csb;
  scp1000_DATAIN = datain;
  scp1000_DATAOUT = dataout;
  scp1000_SCKCLOCK = sckclock;
  
  
  // Pins
  pinMode(scp1000_DRDY, INPUT);
  pinMode(scp1000_CSB, OUTPUT);
  pinMode(scp1000_DATAOUT, OUTPUT);
  pinMode(scp1000_DATAIN, INPUT);
  pinMode(scp1000_SCKCLOCK, OUTPUT);
  digitalWrite(scp1000_CSB, HIGH); //disable device
  digitalWrite(scp1000_DATAOUT, LOW);
  digitalWrite(scp1000_SCKCLOCK, LOW);
  

  // startup device
  boolean success1 = scp1000_startup();
  boolean success2 = scp1000_checksumTest();
  if (success1 == false || success2==false)
    return false;


  // Select operation mode
  if (Operationmode < 0 || Operationmode > 3) Operationmode = 0;
  scp1000_setOperationMode (Operationmode); // Values 0 to 3
  
      
  // Return
  return true;
  
}



//
// Startup SCP1000

boolean scp1000_startup() {

  delay (60); // 60 ms startup delay
  
  for (int i=0;i<6;i++) { // check if startup has finished
   
    int val = scp1000_SPI_read8(Reg_STATUS);
    if (int(1<<Reg_STATUS_startupbit & val) == 0) // Startup successfully
      return true;
    delay (10);
   
  }
 
  return false; // start-up has failed
  
}



//
// Test Checksum

boolean scp1000_checksumTest() {
  
  int checksum = scp1000_SPI_read8(Reg_DATARD8);
  if (Reg_DATARD8_checksumerror == checksum) {// todo
    return false;
  }
    
    
  return true;

}



//
// Set operation mode

void scp1000_setOperationMode(int modeid) {
  
  // Stop active Mode (if SCP1000 is already powerd up and running)
  scp1000_SPI_write(Reg_OPERATION, 0x00);
  delay (50);
  
  
  // Check if DRDY is LOW (If DRDY is HIGH it is necessary to read the output data before activating new measurement mode)
  if (scp1000_DRDY != -1) {
    if (digitalRead(scp1000_DRDY) == HIGH)
      scp1000_SPI_read16(Reg_DATARD16);
  }


  // Select an operation mode
  byte cmd = Reg_OPERATION_modes[modeid]; // Command to write to register (0C)
  scp1000_SPI_write(Reg_OPERATION, cmd);
  
}




//
// scp1000_Read
//   'p' -> 16bit pressure, 't' -> 16bit temperature, 's' -> status, 'o' -> operation mode, 'n' -> operation status

long scp1000_Read(char which) {
  
  // 
  long val;
  switch(which) {
    
  case 'P':case 'p': // 3bit + 16bit pressure
  
    val = scp1000_SPI_read8(Reg_DATARD8);
    val <<= 16;
    unsigned int tmpval;
    tmpval = scp1000_SPI_read16(Reg_DATARD16);
    val += tmpval;
    return val;

  case 'T':case 't': // 16bit temperature

    val = scp1000_SPI_read16(Reg_TEMPOUT);
    
    // if negativ value:
    if (int(1<<13 & val) != 0) {    // bit 13 == 1    
      val ^= 16383;                 // Invert 14 bits (16 bits would be 65535)
      val += 1;                     // plus 1
      val *= -1;
    }
    
    return val;
    break; 

  case 'S':case 's': // status

    return scp1000_SPI_read8(Reg_STATUS);
    break; 

  case 'O':case 'o': // operation mode

    return scp1000_SPI_read8(Reg_OPERATION);
    break; 

  case 'N':case 'n': // operation status

    return scp1000_SPI_read8(Reg_OPSTATUS);
    break; 

  } 
  
}



//
// scp1000_parseDegrees
//   Parse Value to Degres

void scp1000_parseDegrees (int val, int *retSign, int *ret1, int *ret2) {
  
  *retSign = 1;
  if (val < 0)
    *retSign = -1;
  *ret1 = abs(val / temp_factor);
  *ret2 = abs(val  % temp_factor * 100 / temp_factor);
  
}



//
// scp1000_parseDegrees
//   Parse Value to Pascal

long scp1000_parsePa (long val) {
  
  long ret1;
  long ret2;
  scp1000_parsePa (val, &ret1, &ret2);
  return ret1;
  
}
void scp1000_parsePa (long val, long *ret1, long *ret2) {
  
  *ret1 = val / pressure_factor;
  *ret2 = abs(val  % pressure_factor * 100 / pressure_factor);
  
}












// ***************************
//
//  SPI - Functions
//



//
// SPI_write

void scp1000_SPI_write (byte registerByte, byte commandByte) {
  
  digitalWrite(scp1000_CSB, LOW); //turn on device

  // 6 bits register address
  for (int i=5; i>=0; i--){
    
    // write bit
    if (int(1<<i & registerByte) == 0) digitalWrite(scp1000_DATAOUT, LOW); else digitalWrite(scp1000_DATAOUT, HIGH);
    // cycle clock
    digitalWrite(scp1000_SCKCLOCK, HIGH);
    digitalWrite(scp1000_SCKCLOCK, LOW);    

  }
 
  digitalWrite(scp1000_DATAOUT, HIGH); // Indicate that we are Writing
  digitalWrite(scp1000_SCKCLOCK, HIGH);
  digitalWrite(scp1000_SCKCLOCK, LOW);
  digitalWrite(scp1000_DATAOUT, LOW); // LOW
  digitalWrite(scp1000_SCKCLOCK, HIGH);
  digitalWrite(scp1000_SCKCLOCK, LOW);
 
 
  // write 8 bits register content
  for (int i=7; i>=0; i--){

    // write bit
    if (int(1<<i & commandByte) == 0) digitalWrite(scp1000_DATAOUT, LOW); else digitalWrite(scp1000_DATAOUT, HIGH);
    // cycle clock
    digitalWrite(scp1000_SCKCLOCK, HIGH);
    digitalWrite(scp1000_SCKCLOCK, LOW);

  }
  
  
  digitalWrite(scp1000_CSB, HIGH); //turn off device

}



//
// SPI_read8

int scp1000_SPI_read8 (byte registerByte) {

  digitalWrite(scp1000_CSB, LOW); //turn on device

  // 6 bits register address
  for (int i=5; i>=0; i--){
    
    // write bit
    if (int(1<<i & registerByte) == 0) digitalWrite(scp1000_DATAOUT, LOW); else digitalWrite(scp1000_DATAOUT, HIGH);
    // cycle clock
    digitalWrite(scp1000_SCKCLOCK, HIGH);
    digitalWrite(scp1000_SCKCLOCK, LOW);    

  }
 
  digitalWrite(scp1000_DATAOUT, LOW); // Indicate that we are Reading
  digitalWrite(scp1000_SCKCLOCK, HIGH);
  digitalWrite(scp1000_SCKCLOCK, LOW);
  digitalWrite(scp1000_SCKCLOCK, HIGH);
  digitalWrite(scp1000_SCKCLOCK, LOW);

  // read 8 bits register content
  int returnValue = 0;
  for (int i=7; i>=0; i--){

    // read bit
    returnValue += digitalRead (scp1000_DATAIN) << i;
    // cycle clock
    digitalWrite(scp1000_SCKCLOCK, HIGH);
    digitalWrite(scp1000_SCKCLOCK, LOW);    

  }


  digitalWrite(scp1000_CSB, HIGH); //turn off device
  
  return returnValue;

}



//
// SPI_read16

int scp1000_SPI_read16 (byte registerByte) {

  digitalWrite(scp1000_CSB, LOW); //turn on device

  // 6 bits register address
  for (int i=5; i>=0; i--){
    
    // write bit
    if (int(1<<i & registerByte) == 0) digitalWrite(scp1000_DATAOUT, LOW); else digitalWrite(scp1000_DATAOUT, HIGH);
    // cycle clock
    digitalWrite(scp1000_SCKCLOCK, HIGH);
    digitalWrite(scp1000_SCKCLOCK, LOW);

  }

  digitalWrite(scp1000_DATAOUT, LOW); // Indicate that we are Reading
  digitalWrite(scp1000_SCKCLOCK, HIGH);
  digitalWrite(scp1000_SCKCLOCK, LOW);
  digitalWrite(scp1000_SCKCLOCK, HIGH);
  digitalWrite(scp1000_SCKCLOCK, LOW);
 

  // read 8 bits register content
  int returnValue = 0;
  for (int i=15; i>=0; i--){
    
    // read bit
    returnValue += digitalRead (scp1000_DATAIN) << i;
    // cycle clock
    digitalWrite(scp1000_SCKCLOCK, HIGH);
    digitalWrite(scp1000_SCKCLOCK, LOW);    

  }

  digitalWrite(scp1000_CSB, HIGH); //turn off device

  return returnValue;

}



/**************************************************************************************************
*
*  StringSerialProtocol
*
*   Version:      1.1.0 - March 2009
*   Author:       Christoph Wartmann / chair for caad - ETH Z\u00fcrich  /  wartmann[at]arch.ethz.ch
*                 Etienne Ribeiro    / tutorial assistant caad      /  eribeiro[at]ethz.ch
*
*   Desc:         A Protocoll to communicate through the Serial Interface. All data will be
*                 converted to Ascii-Code. This means an int number (2 bytes) now needs up to
*                 6 bytes and must be coded and decoded. On the other hand this protocol is very
*                 reliable even if you reset the board during communication.
*
*   Protocoll:    bits 0 to 8: command
*                 bits 8 to ...: message
*                 last bits: CR, LF or CR LF or LF CR
*                 
*   Methodes:     void Serial_Initialize (int boud)
*                   boud = Boud rate. use 9600
*                 void Serial_setID (char ID)
*                   If this function is called, the specified ID will be sent wenn calling Serial_sendMessage
*                   ID: a value from 0 to 254
*                 void Serial_sendMessage (char cmd, char* message)
*                   Send a string message through serial.
*                   cmd = a char to seperate different commands. Don't use char with ascii 10 and 13
*                   message = string message to send
*                 void Serial_sendMessage (char cmd, int message)
*                   Send an int number through serial.
*                   cmd = a char to seperate different commands. Don't use char with ascii 10 and 13
*                   message = int number to send (-32768 to 32767)
*                 void Serial_sendMessage (char cmd, long message)
*                   Send an long number through serial.
*                   cmd = a char to seperate different commands. Don't use char with ascii 10 and 13
*                   message = long number to send (-2147483648 to 2147483647)
*                 void Serial_check () 
*                   Check if something is coming in. Must be called in LOOP.
*                 int Serial_parseInteger (char* val)
*                   Parse string value to int. (-32768 to 32767)
*                 long Serial_parseLong (char* val)
*                   Parse string value to long. (-2147483648 to 2147483647)
*                 
*   Not implemented:
*      - Sending float and double values
*      - unsigned values
*                 
***************************************************************************************************/ 


// const

const int serial_MAX_COMMAND_LENGTH = 100;
const int serial_LF = 10;
const int serial_CR = 13;


// var

char serial_buffer[serial_MAX_COMMAND_LENGTH]; 
int serial_cBuffer = 0;
int serial_current_bytescount = 0; // length of current Command
char serial_current_command;
//
int serial_ID = 0;



// ------------------
// Functions



// Initialize
void Serial_Initialize (int boud) {
  
  // Serial 
  Serial.begin(boud);
  
}



// Set Id
//   ID: a value from 0 to 254
void Serial_setID (char ID) {
  
  serial_ID = ID;
  
}


// Send Message

void Serial_sendMessage (char cmd, char* message) { if (serial_ID != 0) Serial.print(serial_ID, BYTE); Serial.print(cmd); Serial.println ( message); } // Strings
void Serial_sendMessage (char cmd, char* msg1, char* msg2) { if (serial_ID != 0) Serial.print(serial_ID, BYTE); Serial.print(cmd); Serial.print(msg1); Serial.println (msg2); } // Strings
void Serial_sendMessage (char cmd, char* msg1, int msg2) { if (serial_ID != 0) Serial.print(serial_ID, BYTE); Serial.print(cmd); Serial.print(msg1); Serial.println (msg2, DEC); } // Strings
void Serial_sendMessage (char cmd, int message) { if (serial_ID != 0) Serial.print(serial_ID, BYTE); Serial.print(cmd); Serial.println ( message); }
void Serial_sendMessage (char cmd, int message, int mode) { if (serial_ID != 0) Serial.print(serial_ID, BYTE); Serial.print(cmd); Serial.println ( message, mode); }
void Serial_sendMessage (char cmd, long message) { if (serial_ID != 0) Serial.print(serial_ID, BYTE); Serial.print(cmd); Serial.println ( message); }



// Check if something is coming in. Must be called in loop.

void Serial_check () {

  while (Serial.available() > 0) {
    
    // Fill buffer
    byte val = Serial.read();
    serial_buffer [serial_cBuffer] = val; // store all bytes (excepting first = type)
    
    
    // Message complete
    if (val == serial_LF || val == serial_CR) {
      
      if (serial_cBuffer > 0) {// if cBuffer == 0, no data sent (if you send LF CR or ...)
      
        if (serial_ID == 0 || serial_buffer [0] == serial_ID) { // if ID is not used or first byte = ID
        
          char tmp [serial_cBuffer-3];
          for (int i=1;i<serial_cBuffer;i++)
            tmp [i-1] = serial_buffer[i];
          SerialEvent (serial_buffer[0], tmp);

        }
        
      }
      serial_cBuffer = 0;
      return;
    
    }

    
    // ++
    serial_cBuffer += 1;
    
  }
  
}




// Get length of a char array

int Serial_getLength (char *word) {
  
  int count = 0;
  char *p = word;
  while (*p != '\0') {
    p++;
    count++;
  }
  return count;
  
}





// Parse

int Serial_parseInteger (char* val) {
  
    int ival = 0;
    int sign = 1;
    int len = Serial_getLength(val);
    
  
    for (int i=0;i<len;i++) {
  
      if (val[i] == '-')
        sign = -1;
      if (val[i] >= 48 && val[i] <= 57)
        ival +=  sign * (val[i]-48) * Serial_pow(len-i-1);

    }


    // return
    return ival;
    
}




long Serial_parseLong (char* val) {
  
    long ival = 0;
    int sign = 1;
    int len = Serial_getLength(val);
    
  
    for (int i=0;i<len;i++) {
  
      if (val[i] == '-')
        sign = -1;
      if (val[i] >= 48 && val[i] <= 57)
        ival += sign * (val[i]-48) * Serial_pow(len-i-1);

    }


    // return
    return ival;
    
}


// Serial_pow (there is an issu with pow (10, 2) when looping in wiring 0018, so lets do it from hand)

long Serial_pow (int x) { 
    
  long val = 1;
  for (int i=0;i<x;i++)
    val *= 10;
  return val;
  
}


int main(void)
{
	init();

	setup();
    
	for (;;)
		loop();
        
	return 0;
}

