
/**************************************************************************************************
*
*  SailingSensoreNode
*
*   Version:      1.0.0 - Februar 2009
*   Author:       Christoph Wartmann / chair for caad - ETH Zrich  /  wartmann[at].arch.ethz.ch
*                 Etienne Ribeiro    / tutorial assistant caad      /  eribeiro[at]ethz.ch
*
*   Used Modules: modAneometer: Mesures wind speed
*                 modPressure: Mesures Pressure
*                 modStringSerial: Protocol to transmit values
*
*   Hardware:     Arduino Pro Mini
*                 XBee in RS232 Mode
*                 Pressure Sensor SCP 1000
*                 Aneometer Sensor using induction to calculate wind speed
*
**************************************************************************************************/


#include <EEPROM.h>


//
// Const

int NODE_ID = 16; // 0: not set, id from eeprom will be used (don't use id 10 or 13!)
//
char VERSION[] = "1.0";
#define aneometerSignalIn 9
#define ledPin 13


//
// Var

boolean startupDone;


//
// Setup

void setup() {
      
  // LED (2 * blink)
  pinMode(ledPin, OUTPUT);
  digitalWrite(ledPin, HIGH);
  delay(200);
  digitalWrite(ledPin, LOW);
  delay(200);
  digitalWrite(ledPin, HIGH);


  // NODE_ID to and from EEPROM
  if (NODE_ID == 0)
    NODE_ID = EEPROM.read(10);
  if (EEPROM.read(10) != NODE_ID)
    EEPROM.write (10, NODE_ID);
  
  
  // Serial
  Serial_Initialize (9600);
  Serial_setID (NODE_ID); // ID will be sent
  Serial_sendMessage ('m', "Setup begin");
  Serial_sendMessage ('m', "Firmware Verison ", VERSION);
  Serial_sendMessage ('m', "My ID ", NODE_ID);
  if (NODE_ID == 10 || NODE_ID == 13)
    Serial_sendMessage ('m', " CHANGE ID! ID 10 or 13 is not recommended to use!", VERSION);
  
  
  // Pressure
  //  -> Operationmode: 0: High speed (15bit, 9Hz), 1: High resolution (17bit, 1.8Hz), 2: Ultra low power (15bit, 1Hz), [3 Low poser but triggerd (15bit to 17bit, 1.8Hz) not supported on Sparkfun Breakout]
  byte operationmode = 1;
  startupDone = true;
  if (scp1000_AppendSensor (10, 11, 12, 13, 14, operationmode) == false) {
  
    Serial_sendMessage ('m', "Failed to startup Pressure Sensor");
    startupDone = false;
  
  }
  Serial_sendMessage ('m', "Setup done");

}




//
// Loop

void loop() {

  //
  // Aneometer
  
  
  // Mesure
  // (get Hz)
  int HzCounter = Aneo_getSpeedHZ (aneometerSignalIn, 100);
  // (Parse to m/s and km/h)
  //double ms = Aneo_parseMS (HzCounter);
  Serial_sendMessage ('a', HzCounter);    
  //double kmh = Aneo_parseKMH (HzCounter);
  // (get average Hz)
  //int HzAverage = Aneo_getAverage ();
  //double kmhAverage = Aneo_parseKMH (HzAverage);
  
  
  /*
  // Print
  if (HzCounter > 0) {
    
    // (Hz)
    Serial.print(HzCounter);
    Serial.print(" Hz sec.");
    Serial.print("       ");
    // (m/s)
    Serial.print((int)ms);
    Serial.print(".");
    Serial.print( int(abs((ms - (int)ms) * 100)) );
    Serial.print(" m/s");
    Serial.print("       ");
    // (km/h)
    Serial.print((int)kmh);
    Serial.print(".");
    Serial.print( int(abs((kmh - (int)kmh) * 100)) );
    Serial.print(" km/h");
    Serial.print("       ");
    // 
    Serial.print("Average: ");
    Serial.print((int)kmhAverage);
    Serial.print(".");
    Serial.print( int(abs((kmhAverage - (int)kmhAverage) * 100)) );
    Serial.print(" km/h");
    //
    Serial.println("");
    
  }
*/

  //
  // Pressure
  
  if (startupDone) {
    
    // Temperature
    int sign;
    int temp1;
    int temp2;
    scp1000_parseDegrees(scp1000_Read('t'), &sign, &temp1, &temp2);
    long temperature = sign * (temp1 * 100 + temp2);
    Serial_sendMessage ('t', temperature);
    long pressure = scp1000_parsePa(scp1000_Read('p'));
    Serial_sendMessage ('p', pressure);    
 
  }
  
   
  // Take a break
  delay(100);
  
}





// SerialEvent

void SerialEvent (char cmd, char* message) {
  
  
}
